
/*******************************************************************************
* Copyright (c) 2017 Olivier Langella <olivier.langella@u-psud.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <olivier.langella@u-psud.fr> - initial API and implementation
******************************************************************************/

#include "proteinmatch.h"
#include <pappsomspp/msrun/msrunid.h>
#include <pappsomspp/grouping/grpprotein.h>
#include <set>


ProteinMatch::ProteinMatch()
{

}

ProteinMatch::~ProteinMatch()
{
    auto it = _peptide_match_list.begin();
    while (it != _peptide_match_list.end()) {
        delete (*it);
        it++;
    }
}

void ProteinMatch::updateAutomaticFilters(const AutomaticFilterParameters & automatic_filter_parameters) {
    _proxy_valid = false;
    unsigned int _number_of_valid_peptides =0;
    std::map<const pappso::MsRunId*, unsigned int> _count_per_msrun;
    bool cross_sample = automatic_filter_parameters.getFilterCrossSamplePeptideNumber();

    for (auto & p_peptide_match : _peptide_match_list) {
        p_peptide_match->updateAutomaticFilters(automatic_filter_parameters);
        //count valid and checked peptides :
        if (p_peptide_match->isValidAndChecked()) {
            if (cross_sample) {
                _number_of_valid_peptides++;
            }
            else {
                std::pair<std::map<const pappso::MsRunId*, unsigned int>::iterator,bool> ret = _count_per_msrun.insert(std::pair<const pappso::MsRunId*, unsigned int>(p_peptide_match->getMsRunIdSp().get(),0));
                ret.first->second +=1;
                if (ret.first->second > _number_of_valid_peptides) {
                    _number_of_valid_peptides = ret.first->second;
                }
            }
        }
    }


    //update protein evalue ?


    if (_number_of_valid_peptides >= automatic_filter_parameters.getFilterMinimumPeptidePerMatch()) {
        if (_evalue <= automatic_filter_parameters.getFilterProteinEvalue()) {
            _proxy_valid = true;
        }
    }
}

const ProteinXtpSp & ProteinMatch::getProteinXtpSp() const {
    return _protein_sp;
}
void ProteinMatch::setEvalue(pappso::pappso_double evalue) {
    _evalue = evalue;
}

pappso::pappso_double ProteinMatch::getEvalue() const {
    return _evalue;
}
void ProteinMatch::setProteinXtpSp(ProteinXtpSp protein_sp) {
    _protein_sp = protein_sp;
}


bool ProteinMatch::isValid() const {
    return _proxy_valid;
}
bool ProteinMatch::isChecked() const {
    return _checked;
}

bool ProteinMatch::isValidAndChecked() const {
    return _proxy_valid && _checked;
}

void ProteinMatch::setChecked(bool arg1) {
    _checked = arg1;
}
void ProteinMatch::addPeptideMatch(PeptideMatch * peptide_match) {
    _peptide_match_list.push_back(peptide_match);
}


std::vector<PeptideMatch *> & ProteinMatch::getPeptideMatchList() {
    return _peptide_match_list;
}

const std::vector<PeptideMatch *> & ProteinMatch::getPeptideMatchList() const {
    return _peptide_match_list;
}

void ProteinMatch::setGroupingExperiment(GroupingExperiment * p_grp_experiment) {
    _sp_grp_protein = nullptr;
    if (isValidAndChecked()) {
        _sp_grp_protein = p_grp_experiment->getGrpProteinSp(this);

        for (auto & p_peptide_match : _peptide_match_list) {
            p_peptide_match->setGrpPeptideSp(nullptr);
            if (p_peptide_match->isValidAndChecked()) {
                p_peptide_match->setGrpPeptideSp(p_grp_experiment->setGrpPeptide(_sp_grp_protein, p_peptide_match));
            }
        }


        if (_protein_sp.get()->isContaminant()) {
            p_grp_experiment->addPostGroupingGrpProteinSpRemoval(_sp_grp_protein);
        }

    }
}

const pappso::GrpProteinSp & ProteinMatch::getGrpProteinSp() const {
    return _sp_grp_protein;
}

size_t ProteinMatch::countValidSpectrum()const {
    return std::count_if (_peptide_match_list.begin(), _peptide_match_list.end(), [](const PeptideMatch * p_peptide_match) {
        return (p_peptide_match->isValid());
    });
}

size_t ProteinMatch::countValidAndCheckedSpectrum()const {
    return std::count_if (_peptide_match_list.begin(), _peptide_match_list.end(), [](const PeptideMatch * p_peptide_match) {
        return (p_peptide_match->isValidAndChecked());
    });
}
size_t ProteinMatch::countUniqueSequence()const {
    std::set<QString> sequence_list;
    for (auto & p_peptide_match : _peptide_match_list) {
        if (p_peptide_match->isValidAndChecked()) {
            sequence_list.insert(p_peptide_match->getPeptideXtpSp().get()->getSequence());
        }
    }
    return sequence_list.size();
}

pappso::pappso_double ProteinMatch::getCoverage() const {
    size_t prot_size = _protein_sp.get()->size();
    //qDebug() << "ProteinMatch::getCoverage begin prot_size=" << prot_size << " " << _protein_sp.get()-//>getSequence();
    if (prot_size == 0) return 0;
    bool cover_bool[prot_size] = {false};
    for (auto & p_peptide_match : _peptide_match_list) {
        if (p_peptide_match->isValidAndChecked()) {
            size_t size = p_peptide_match->getPeptideXtpSp().get()->size();
            size_t offset = p_peptide_match->getStart();
            if (offset >= 0) {
                for (size_t i=0; (i < size) && (offset < prot_size) ; i++,offset++) {
                    cover_bool[offset] = true;
                }
            }
        }
    }
    size_t count = 0;
    for (size_t i=0; i < prot_size; i++) {
        if(cover_bool[i]) count++;
    }
    //qDebug() << "ProteinMatch::getCoverage count=" << count << " prot_size=" << prot_size;
    return (((pappso::pappso_double)count)/ ((pappso::pappso_double)prot_size));
}
const GroupingGroupSp & ProteinMatch::getGroupingGroupSp() const {
  return _sp_group;
}

void ProteinMatch::setGroupInstance(GroupStore & group_store) {
    _sp_group = nullptr;
    if (_sp_grp_protein != nullptr) {
        unsigned int group_number = _sp_grp_protein.get()->getGroupNumber();
        if (group_number > 0) {
            _sp_group = group_store.getInstance(group_number);
            _sp_group.get()->add(this);
        }
    }
}
